#!/bin/python3

import sys
import os
import shutil
import re
import tempfile
import subprocess


def _print(msg):
    print('[gomacc_subs] {}'.format(msg))


def main():
    _print('Interpreter ver: {}.{}'.format(sys.version_info[0], sys.version_info[1]))
    if sys.version_info[0] < 2:
        _print('Interpreter must be 3.0 and above')
        sys.exit(1)

    rulesPath = os.path.abspath(sys.argv[1])
    _print('Rules file path [{}]'.format(rulesPath))
    if not os.path.isfile(rulesPath):
        _print("File path {} does not exist".format(rulesPath))
        sys.exit(1)

    proc = subprocess.Popen('where gomacc', stdout=subprocess.PIPE)
    exitCode = proc.wait()
    if exitCode != 0:
        _print("Cannot find 'gomacc', add to the path")
        sys.exit(1)
    for line in proc.stdout:
        output = str(line.rstrip()).replace('b\'', '').replace('\'', '')

    gomaExecPath = os.path.abspath(output)
    _print('gomacc executable file path [{}]'.format(gomaExecPath))
    if not os.path.isfile(gomaExecPath):
        _print("File path {} does not exist".format(gomaExecPath))
        sys.exit(1)

    gomaExecPathUnix = gomaExecPath.replace("\\", "/")

    inputFilePath = rulesPath

    # create backup
    inputFileBackupPath = inputFilePath + ".bak"
    shutil.copyfile(inputFilePath, inputFileBackupPath)

    fh, tempFilePath = tempfile.mkstemp()
    with os.fdopen(fh, "w") as outputFile:
        with open(inputFilePath) as inputFile:
            counter = 0
            # probably not working in all cases
            execExp = "[^ ]+cl.exe"
            execChangeTo = gomaExecPathUnix + " cl"
            commandExp = "command = \"?{}\"?"
            for line in inputFile:
                if bool(re.search(commandExp.format(execExp), line)):
                    transformed = re.sub(execExp, execChangeTo, line)
                    outputFile.write(transformed)
                    _print('Change {}'.format(line))
                    _print('>>>>>> {}'.format(transformed))
                    counter += 1
                else:
                    outputFile.write(line)
            _print('{} substitutions done'.format(counter))

    # Copy the file permissions from the old file to the new file
    shutil.copymode(inputFilePath, tempFilePath)
    # Remove original file
    os.remove(inputFilePath)
    # Move new file
    shutil.move(tempFilePath, inputFilePath)

    sys.exit(0)


if __name__ == '__main__':
    main()
